@@ -5,8 +5,8 @@
 ;; Author: Earl Hyatt
 ;; Created: November 2020
 ;; URL: https://github.com/okamsn/loopy
-;; Package-Version: 20251013.154
-;; Package-Revision: 9b6b5c09849c
+;; Package-Version: 20251116.13
+;; Package-Revision: 07a9952cd6c0
 ;; Package-Requires: ((emacs "28.1") (map "3.3.1") (seq "2.22") (compat "29.1.3") (stream "2.4.0"))
 ;; Keywords: extensions
 ;; LocalWords:  Loopy's emacs Edebug
@@ -364,7 +364,12 @@ The function creates quoted code that should be used by a macro."
                       ;; completes to see whether we should return
                       ;; an early return value (the value of the `cl-block')
                       ;; or a possibly now-updated value of `loopy-result'.
-                      ,(when loopy--final-do
+                      ;;
+                      ;; We don't need to check whether it completed
+                      ;; when there is a `finally-return' because we
+                      ;; know that we are not using an implied return value.
+                      ,(when (and loopy--final-do
+                                  (not loopy--final-return))
                          `(setq ,loop-completed-var t))
                       ;; This can be overridden by any call to
                       ;; `cl-return-from'.
